@@ -1,6 +1,6 @@
 """
 
-Copyright (C) 2020 Vanessa Sochat.
+Copyright (C) 2020-2022 Vanessa Sochat.
 
 This Source Code Form is subject to the terms of the
 Mozilla Public License, v. 2.0. If a copy of the MPL was not distributed
@@ -20,16 +20,21 @@ def main(args, extra):
     # Provide shell if available
     shell = RSE_SHELL.lower()
     if shell in lookup:
-        return lookup[shell](args)
+        try:
+            return lookup[shell](args)
+        except ImportError:
+            pass
 
     # Otherwise present order of liklihood to have on system
     for shell in shells:
-        return lookup[shell](args)
+        try:
+            return lookup[shell](args)
+        except ImportError:
+            pass
 
 
 def ipython(args):
-    """give the user an ipython shell, optionally with an endpoint of choice.
-    """
+    """give the user an ipython shell, optionally with an endpoint of choice."""
     client = Encyclopedia(config_file=args.config_file)
     assert client
     from IPython import embed
