@@ -1,6 +1,6 @@
 """
 
-Copyright (C) 2020 Vanessa Sochat.
+Copyright (C) 2020-2022 Vanessa Sochat.
 
 This Source Code Form is subject to the terms of the
 Mozilla Public License, v. 2.0. If a copy of the MPL was not distributed
@@ -19,9 +19,11 @@ from rse.utils.command import get_github_username
 from rse.utils.urls import repository_regex
 from rse.main.parsers import get_parser
 from rse.main.criteria import get_criteria
+import rse.main.export
 from rse.main.taxonomy import get_taxonomy
 from rse.logger.message import bot as message
 
+import json
 import logging
 import os
 import re
@@ -31,28 +33,31 @@ parser_regex = "github"
 
 
 class Encyclopedia:
-    """An encyclopedia is one or more namespaces to store research
-       software. By default, we create a structure on the filesystem,
-       however an sqlite database (or other) can be used.
+    """
+    An encyclopedia is one or more namespaces to store research
+    software. By default, we create a structure on the filesystem,
+    however an sqlite database (or other) can be used.
     """
 
     def __init__(self, config_file=None, database=None, generate=False):
-        """create a software repository. We take a config file, which should
-           sit at the root of the repository, and then parse the subfolders
-           accordingly.
+        """
+        create a software repository. We take a config file, which should
+        sit at the root of the repository, and then parse the subfolders
+        accordingly.
         """
         self.config = Config(config_file or RSE_CONFIG_FILE, generate=generate)
         self.config_dir = os.path.dirname(self.config.configfile)
         self.initdb(database)
 
     def initdb(self, database):
-        """setup the rse home (where the config directory is stored) and the
-           database specification. If a database string is required (and not
-           provided) alert the user and exit on error).
+        """
+        setup the rse home (where the config directory is stored) and the
+        database specification. If a database string is required (and not
+        provided) alert the user and exit on error).
 
-           Arguments:
-            - config_dir (str) : the configuration directory (home for rse)
-            - database (str) : a string to specify the database setup
+        Arguments:
+         - config_dir (str) : the configuration directory (home for rse)
+         - database (str) : a string to specify the database setup
         """
         self.database = (
             database
@@ -61,7 +66,7 @@ class Encyclopedia:
             or "filesystem"
         )
         database_string = self.config.get("DEFAULT", "databaseconnect")
-        bot.info("Database: %s" % self.database)
+        bot.debug("Database: %s" % self.database)
 
         # Supported database options
         valid = ("sqlite", "postgresql", "mysql+pymysql", "filesystem")
@@ -80,35 +85,40 @@ class Encyclopedia:
         )
 
     def exists(self, uid):
-        """based on a parser type and unique identifier, determine if software
-           exists in the database
+        """
+        based on a parser type and unique identifier, determine if software
+        exists in the database
         """
         parser = get_parser(uid, config=self.config)
         return self.db.exists(parser.uid)
 
     def list(self, name=None):
-        """A wrapper to the database list_repos function. Optionally take
-           a whole parser name (e.g., github) or just a specific uid. No
-           parser indicates that we list everything.
+        """
+        A wrapper to the database list_repos function. Optionally take
+        a whole parser name (e.g., github) or just a specific uid. No
+        parser indicates that we list everything.
         """
         return self.db.list_repos(name)
 
     def list_criteria(self):
-        """Get a listing of criteria from the rse API
+        """
+        Get a listing of criteria from the rse API
         """
         if not hasattr(self, "criteria"):
             self.criteria = get_criteria()
         return self.criteria
 
     def list_taxonomy(self):
-        """Get a listing of a flattened taxonomy from the rse API
+        """
+        Get a listing of a flattened taxonomy from the rse API
         """
         if not hasattr(self, "taxonomy"):
             self.taxonomy = get_taxonomy()
         return self.taxonomy
 
     def bulk_add(self, filename):
-        """Given a filename with a single list of repos, add each
+        """
+        Given a filename with a single list of repos, add each
         """
         repos = []
         if os.path.exists(filename):
@@ -118,7 +128,8 @@ class Encyclopedia:
         return repos
 
     def bulk_update(self, filename, rewrite=False):
-        """Given a filename with a single list of repos, add each
+        """
+        Given a filename with a single list of repos, add each
         """
         repos = []
         if os.path.exists(filename):
@@ -130,18 +141,20 @@ class Encyclopedia:
                     pass
         return repos
 
-    def add(self, uid, quiet=False):
-        """A wrapper to add a repository to the software database.
+    def add(self, uid, quiet=False, data=None):
+        """
+        A wrapper to add a repository to the software database.
         """
         if not self.exists(uid):
-            repo = self.db.add(uid)
+            repo = self.db.add(uid, data=data)
             return repo
         if not quiet:
             bot.error(f"{uid} already exists in the database.")
 
     def get(self, uid=None):
-        """A wrapper to get a repo id from the database. If an id is not provided,
-           will return the last updated repo based on timestamp of file or database.
+        """
+        A wrapper to get a repo id from the database. If an id is not provided,
+        will return the last updated repo based on timestamp of file or database.
         """
         return self.db.get(uid)
 
@@ -149,8 +162,9 @@ class Encyclopedia:
         return self.db.get_or_create(uid)
 
     def clear(self, target=None, noprompt=False):
-        """clear takes a target, and that can be a uid, parser, or none
-           We ask the user for confirmation.
+        """
+        clear takes a target, and that can be a uid, parser, or none
+        We ask the user for confirmation.
         """
         # Case 1: no target indicates clearing all
         if not target:
@@ -176,24 +190,27 @@ class Encyclopedia:
         else:
             raise RuntimeError(f"Unrecognized {target} to clear")
 
-    def update(self, uid, rewrite=False):
-        """Update an existing software repository.
+    def update(self, uid, rewrite=False, data=None):
+        """
+        Update an existing software repository.
         """
         try:
             repo = self.get(uid)
-            self.db.update(repo, rewrite=rewrite)
+            self.db.update(repo, rewrite=rewrite, data=data)
             bot.info(f"{repo.uid} has been updated.")
             return repo
         except RepoNotFoundError:
             bot.error(f"{uid} does not exist.")
 
-    def label(self, uid, key, value, force=False):
-        """Update an existing software repository with a specific label.
+    def label(self, uid, key, value, force=False, quiet=False):
+        """
+        Update an existing software repository with a specific label.
         """
         try:
             repo = self.get(uid)
             self.db.label(repo, key, value, force=force)
-            bot.info(f"{repo.uid} has been updated.")
+            if not quiet:
+                bot.info(f"{repo.uid} has been updated.")
             return repo
         except RepoMetadataExistError:
             bot.error(
@@ -202,21 +219,80 @@ class Encyclopedia:
         except RepoNotFoundError:
             bot.error(f"{uid} does not exist.")
 
-    def search(self, query):
-        """Search across commands and general metadata for a string of interest.
-           We use regular expressions (re.search) so they are supported.
-           Search is only available for non-filesystem databases.
+    def search(self, query, taxonomy=None, criteria=None):
         """
-        results = self.db.search(query)
+        Search across commands and general metadata for a string of interest.
+        We use regular expressions (re.search) so they are supported.
+        Search is only available for non-filesystem databases.
+        """
+        results = self.db.search(query, taxonomy=taxonomy, criteria=criteria)
         if results:
             return results
         bot.info(f"No results matching {query}")
 
+    # Yield Repos (with full metadata)
+
+    def yield_repos(self):
+        """
+        yield repos, one at a time.
+        """
+        for name in self.list():
+            # The parser will provide consistent handles to avatar, description, etc.
+            repo = self.get(name[0])
+
+            # Relational needs to load from string
+            data = repo.data
+            if isinstance(data, str):
+                data = json.loads(data)
+            parser = get_parser(repo.uid, config=self.config)
+            parser.load(data)
+            # Data includes the uid and common variables
+            yield parser.export()
+
+    # Topics
+
+    def topics(self, pattern=None):
+        """
+        return a list of unique topics, optionally matching a pattern
+        """
+        topics = set()
+        for name in self.list():
+            repo = self.get(name[0])
+
+            # Relational needs to load from string
+            data = repo.data
+            if isinstance(data, str):
+                data = json.loads(data)
+
+            # Get the list of topics, optionally filter by a pattern
+            topiclist = data.get("topics", []) or data.get("data", {}).get("topics", [])
+            if pattern is not None:
+                topiclist = [t for t in topiclist if re.search(pattern, t)]
+
+            # Add to topics set
+            [topics.add(t) for t in topiclist]
+        return sorted(list(topics))
+
+    def repos_by_topics(self, topics):
+        """
+        return a list of unique topics, optionally matching a pattern
+        """
+        repos = []
+        for name in self.list():
+            repo = self.get(name[0])
+            data = repo.parser.get_metadata()
+            topiclist = data.get("topics", []) or data.get("data", {}).get("topics", [])
+            if set(topics).intersection(set(topiclist)):
+                repos.append(repo.uid)
+
+        return sorted(repos)
+
     # Save Handlers
 
     def save_criteria(self, repo):
-        """Given a repository that can be a handle to a filesystem entry
-           or database, save the criteria
+        """
+        Given a repository that can be a handle to a filesystem entry
+        or database, save the criteria
         """
         # The filesystem database saves at the end
         if hasattr(repo, "save_criteria"):
@@ -227,8 +303,9 @@ class Encyclopedia:
             self.db.update(repo)
 
     def save_taxonomy(self, repo, username, uids):
-        """Given a repository that can be a handle to a filesystem entry
-           or database, save the criteria
+        """
+        Given a repository that can be a handle to a filesystem entry
+        or database, save the criteria
         """
         if hasattr(repo, "save_taxonomy"):
             repo.taxonomy[username] = uids
@@ -246,8 +323,9 @@ class Encyclopedia:
         criteria_uids=None,
         include_empty=False,
     ):
-        """analyze takes a repository and calculates a "final answer" based on user provided
-           thresholds
+        """
+        analyze takes a repository and calculates a "final answer" based on user provided
+        thresholds
         """
         results = []
         for repo in self.list():
@@ -266,8 +344,9 @@ class Encyclopedia:
     def analyze(
         self, repo, cthresh=0.5, tthresh=1, taxonomy_uids=None, criteria_uids=None
     ):
-        """analyze takes a repository and calculates a "final answer" based on user provided
-           thresholds
+        """
+        analyze takes a repository and calculates a "final answer" based on user provided
+        thresholds
         """
         # If taxonomy or criteria lists aren't defined, use all
         if not taxonomy_uids:
@@ -312,9 +391,18 @@ class Encyclopedia:
 
         return metrics
 
+    def export(self, path, exporter="jekyll"):
+        """
+        Export rsepedia to a path. If it does not exist, we start with template.
+        """
+        path = os.path.abspath(path)
+        exporter = rse.main.export.get_exporter(exporter)(path)
+        return exporter.export(repos=self.yield_repos())
+
     def summary(self, repo=None):
-        """Summarize metrics for the entire database if uid is not defined,
-           or one specific repository.
+        """
+        Summarize metrics for the entire database if uid is not defined,
+        or one specific repository.
         """
         if repo is None:
             repos = self.list()
@@ -384,14 +472,15 @@ class Encyclopedia:
     # Annotation
 
     def annotate(self, username, atype, unseen_only=True, repo=None, save=False):
-        """Annotate the encyclopedia, either for criteria or taxonomy.
-           A username is required for the namespace.
- 
-           Arguments:
-            - username (str) : the user's GitHub username
-            - atype (str) : the annotation type
-            - unseen_only (bool): annotate only items not seen by username
-            - repo (str) : annotate a particular software repository
+        """
+        Annotate the encyclopedia, either for criteria or taxonomy.
+        A username is required for the namespace.
+
+        Arguments:
+         - username (str) : the user's GitHub username
+         - atype (str) : the annotation type
+         - unseen_only (bool): annotate only items not seen by username
+         - repo (str) : annotate a particular software repository
         """
         # git config user.name
         if not username:
@@ -404,8 +493,9 @@ class Encyclopedia:
         bot.error(f"Unknown annotation type, {atype}.")
 
     def _import_annotation(self, input_file, username, stop_line="## Criteria"):
-        """A general helper (private)  function to import an annotation, meaning
-           we parse a repository and return additional lines for parsing.
+        """
+        A general helper (private)  function to import an annotation, meaning
+        we parse a repository and return additional lines for parsing.
         """
         if not username or not input_file:
             raise RuntimeError(
@@ -434,10 +524,11 @@ class Encyclopedia:
         return repo, lines
 
     def import_criteria_annotation(self, input_file, username):
-        """Given a text file that has a bullet list of (some checked) criteria
-           as might be generated in a GitHub issue, read in the file and the
-           username to do an annotation. If a user has already done an annotation,
-           his or her record is updated.
+        """
+        Given a text file that has a bullet list of (some checked) criteria
+        as might be generated in a GitHub issue, read in the file and the
+        username to do an annotation. If a user has already done an annotation,
+        his or her record is updated.
         """
         repo, lines = self._import_annotation(input_file, username)
 
@@ -455,10 +546,11 @@ class Encyclopedia:
         self.save_criteria(repo)
 
     def import_taxonomy_annotation(self, input_file, username):
-        """Given a text file that has a bullet list of (some checked) criteria
-           as might be generated in a GitHub issue, read in the file and the
-           username to do an annotation. If a user has already done an annotation,
-           his or her record is updated.
+        """
+        Given a text file that has a bullet list of (some checked) criteria
+        as might be generated in a GitHub issue, read in the file and the
+        username to do an annotation. If a user has already done an annotation,
+        his or her record is updated.
         """
         repo, lines = self._import_annotation(
             input_file, username, stop_line="## Taxonomy"
@@ -476,8 +568,9 @@ class Encyclopedia:
         self.save_taxonomy(repo, username, uids)
 
     def yield_criteria_annotation_repos(self, username, unseen_only=True, repo=None):
-        """Given a username, repository, and preference for seen / unseen,
-           yield a repository to annotate.
+        """
+        Given a username, repository, and preference for seen / unseen,
+        yield a repository to annotate.
         """
         if repo is None:
             repos = self.list()
@@ -498,8 +591,9 @@ class Encyclopedia:
                     yield repo, item
 
     def yield_taxonomy_annotation_repos(self, username, unseen_only=True, repo=None):
-        """Given a username, repository, and preference for seen / unseen,
-           yield a repository to annotate.
+        """
+        Given a username, repository, and preference for seen / unseen,
+        yield a repository to annotate.
         """
         if repo is None:
             repos = self.list()
@@ -517,10 +611,11 @@ class Encyclopedia:
                 yield repo
 
     def annotate_criteria(self, username, unseen_only=True, repo=None, save=False):
-        """Annotate criteria, meaning we iterate over repos and criteria that 
-           match the user request, namely to annotate unseen only, or just
-           a particular repository. If the repository is specified, unseen_only
-           is assumed False.
+        """
+        Annotate criteria, meaning we iterate over repos and criteria that
+        match the user request, namely to annotate unseen only, or just
+        a particular repository. If the repository is specified, unseen_only
+        is assumed False.
         """
         annotations = {}
         last = None
@@ -562,10 +657,11 @@ class Encyclopedia:
         return annotations
 
     def annotate_taxonomy(self, username, unseen_only=True, repo=None, save=False):
-        """Annotate taxonomy, meaning we iterate over repos and criteria that 
-           match the user request, namely to annotate unseen only, or just
-           a particular repository. If the repository is specified, unseen_only
-           is assumed False.
+        """
+        Annotate taxonomy, meaning we iterate over repos and criteria that
+        match the user request, namely to annotate unseen only, or just
+        a particular repository. If the repository is specified, unseen_only
+        is assumed False.
         """
         annotations = {}
 
