@@ -1,6 +1,6 @@
 """
 
-Copyright (C) 2020 Vanessa Sochat.
+Copyright (C) 2020-2022 Vanessa Sochat.
 
 This Source Code Form is subject to the terms of the
 Mozilla Public License, v. 2.0. If a copy of the MPL was not distributed
@@ -18,8 +18,7 @@ parser_regex = "github"
 
 
 def get_criteria():
-    """Get criteria from the default endpoint.
-    """
+    """Get criteria from the default endpoint."""
     response = requests.get(f"{RSE_API_ENDPOINT}/criteria/")
     if response.status_code != 200:
         sys.exit(f"Problem with getting {RSE_API_ENDPOINT}/criteria/")
