@@ -1,6 +1,6 @@
 """
 
-Copyright (C) 2020 Vanessa Sochat.
+Copyright (C) 2020-2022 Vanessa Sochat.
 
 This Source Code Form is subject to the terms of the
 Mozilla Public License, v. 2.0. If a copy of the MPL was not distributed
@@ -17,16 +17,18 @@ bot = logging.getLogger("rse.main.database.sqlite")
 
 
 class SqliteDatabase(RelationalDatabase):
-    """A SqliteDatabase writes to a rse.db file in $HOME/.rse. This is
-       the suggested database backend for QueueMe, as it doesn't require anything
-       beyond a filesystem and still allows for relational type queries.
+    """
+    A SqliteDatabase writes to a rse.db file in $HOME/.rse. This is
+    the suggested database backend for QueueMe, as it doesn't require anything
+    beyond a filesystem and still allows for relational type queries.
     """
 
     database = "sqlite"
 
     def __init__(self, config_dir, config=None, **kwargs):
-        """init for the filesystem ensures that the base folder (named 
-           according to the studyid) exists.
+        """
+        init for the filesystem ensures that the base folder (named
+        according to the studyid) exists.
         """
         database_file = kwargs.get("database_string", "rse.db") or "rse.db"
 
