@@ -1,7 +1,6 @@
-#!/usr/bin/env python
 """
 
-Copyright (C) 2020 Vanessa Sochat.
+Copyright (C) 2020-2022 Vanessa Sochat.
 
 This Source Code Form is subject to the terms of the
 Mozilla Public License, v. 2.0. If a copy of the MPL was not distributed
@@ -9,13 +8,10 @@ with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
 """
 
-import os
-import sys
-import pytest
-
 
 def test_parser_gitlab(tmp_path):
-    """Test the gitlab parser.
+    """
+    Test the gitlab parser.
     """
     from rse.main.parsers import GitLabParser
 
